/*
 * Created on 14/04/2004
 *
 */
package infinispan.test;

import java.util.HashMap;
import java.util.Map;
import java.util.Set;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.slf4j.MDC;




/**
 *
 * LogUtils encapsulated all nessecary logging operations.
 * this class thread safety relies on the underlying Logging implementation thread safety
 * NOTE: commons-logging LogFactory retrieval and Log retrieval are not
 *       expensive operations. they occour only once per LogFactory/Log
 *       so that they can be called multiple times with no overhead
 *
 * @author Guy Katz
 * @version 1.0
 * @since 1.0
 */
public class LogUtils {

	private static HashMap<String, Logger> logMap = new HashMap<String, Logger>(); 
	
	/**
	 * constructor - never called directly
	 */
	private LogUtils() {} 
	
	/**
	 * @param callingClass is the reflection of the calling class
	 * @return a Log object
	 */
	private static Logger getLogger(Class callingClass) {
		Logger log = logMap.get(callingClass.getName());
		if (log == null)
		{
			log = LoggerFactory.getLogger(callingClass);
			logMap.put(callingClass.getName(), log);			
		}
		return log;
	}
	
	
	/**
	 * @param callingClass is the reflection of the calling class
	 * @return true if debug is enabled, false otherwise
	 */
	public static boolean isDebugEnabled(Class callingClass)
	{
		Logger logger = getLogger(callingClass);
		return logger.isDebugEnabled();		
	}
	
	/**
	 * @param callingClass is the reflection of the calling class
	 * @return true if trace is enabled, false otherwise
	 */
	public static boolean isTraceEnabled(Class callingClass)
	{
		Logger logger = getLogger(callingClass);
		return logger.isTraceEnabled();
	}
	
	/**
	 * log a message with a trace priority
	 * @param callingClass the Class object of the calling code
	 * @param msg the message to be logged
	 */
	public static void trace(Class callingClass, String msg){
		trace(callingClass, msg, null);
	}
	
	/**
	 * log a message and a throwable object with a trace priority
	 * @param callingClass the Class object of the calling code
	 * @param msg the message to be logged
	 * @param throwable an exception or error that needs to be logged
	 */
	public static void trace(Class callingClass, String msg, Throwable throwable){
		Logger log = getLogger(callingClass);
		if(log.isTraceEnabled()){
			log.trace(msg, throwable);
		}			
	}
	
	/**
	 * log a message with a debug priority
	 * @param callingClass the Class object of the calling code
	 * @param msg the message to be logged
	 */
	public static void debug(Class callingClass, String msg){
		debug(callingClass, msg, null);
	}
	
	/**
	 * log a message and a throwable object with a debug priority
	 * @param callingClass the Class object of the calling code
	 * @param msg the message to be logged
	 * @param throwable an exception or error that needs to be logged
	 */
	public static void debug(Class callingClass, String msg, Throwable throwable){
		Logger log = getLogger(callingClass);
		if(log.isDebugEnabled()){
			log.debug(msg, throwable);
		}			
	}
	
	/**
	 * log a message with a info priority
	 * @param callingClass the Class object of the calling code
	 * @param msg the message to be logged
	 */
	public static void info(Class callingClass, String msg){
		info(callingClass, msg, null);
	}
	
	/**
	 * log a message and a throwable object with a info priority
	 * @param callingClass the Class object of the calling code
	 * @param msg the message to be logged
	 * @param throwable an exception or error that needs to be logged
	 */
	public static void info(Class callingClass, String msg, Throwable throwable){
		Logger log = getLogger(callingClass);
		
		if(log.isInfoEnabled()){
			log.info(msg, throwable);
		}			
	}
	
	/**
	 * log a message with a warn priority
	 * @param callingClass the Class object of the calling code
	 * @param msg the message to be logged
	 */
	public static void warn(Class callingClass, String msg){
		warn(callingClass, msg, null);
	}
	
	/**
	 * log a message and a throwable object with a warn priority
	 * @param callingClass the Class object of the calling code
	 * @param msg the message to be logged
	 * @param throwable an exception or error that needs to be logged
	 */
	public static void warn(Class callingClass, String msg, Throwable throwable){
		Logger log = getLogger(callingClass);
		if(log.isWarnEnabled()){
			log.warn(msg, throwable);
		}			
	}

	/**
	 * log a message with a error priority
	 * @param callingClass the Class object of the calling code
	 * @param msg the message to be logged
	 */
	public static void error(Class callingClass, String msg){
		error(callingClass, msg, null);
	}
	
	/**
	 * log a message and a throwable object with a error priority
	 * @param callingClass the Class object of the calling code
	 * @param msg the message to be logged
	 * @param throwable an exception or error that needs to be logged
	 */
	public static void error(Class callingClass, String msg, Throwable throwable){
		Logger log = getLogger(callingClass);
		if(log.isErrorEnabled()){
			log.error(msg, throwable);
		}			
	}
	
	/**
	 * log a message with a fatal priority
	 * @param callingClass the Class object of the calling code
	 * @param msg the message to be logged
	 */
	public static void fatal(Class callingClass, String msg){
		fatal(callingClass, msg, null);
	}
	
	/**
	 * log a message and a throwable object with a fatal priority
	 * @param callingClass the Class object of the calling code
	 * @param msg the message to be logged
	 * @param throwable an exception or error that needs to be logged
	 */
	public static void fatal(Class callingClass, String msg, Throwable throwable){
		Logger log = getLogger(callingClass);		
			log.error(msg, throwable);
	}
	
	/**
	 * Set the current thread MDC with the given session parameters, in order to see them printed in log files.
	 * @param sessionParams map of session parameters 
	 */
	public static void addParamsToMDC(Map<String, String> sessionParams) {
		if (sessionParams != null) {
			MDC.setContextMap(sessionParams);
		}
	}
	
	/**
	 * Remove given session parameters from MDC, so they won't be printed in log files.
	 * @param sessionParams
	 */
	public static void removeParamsFromMDC(Map<String, String> sessionParams) {
		Set<String> mapKeys = null;		
		if (sessionParams != null) { 
			// Remove all session parameters from MDC 
			mapKeys = sessionParams.keySet();
			for (String key:mapKeys) {
				MDC.remove(key);
			}
		}		
	}
}
